package com.tuoheng.admin.service.accident.query;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tuoheng.admin.conver.AccidentConverMapper;
import com.tuoheng.admin.conver.InspectionConverMapper;
import com.tuoheng.admin.entity.Accident;
import com.tuoheng.admin.entity.Dept;
import com.tuoheng.admin.entity.Inspection;
import com.tuoheng.admin.entity.User;
import com.tuoheng.admin.enums.AccidentEnum;
import com.tuoheng.admin.enums.MarkEnum;
import com.tuoheng.admin.enums.RoleEnum;
import com.tuoheng.admin.enums.code.inspection.QueryInspectionPageListCodeEnum;
import com.tuoheng.admin.mapper.AccidentMapper;
import com.tuoheng.admin.mapper.DeptMapper;
import com.tuoheng.admin.mapper.InspectionMapper;
import com.tuoheng.admin.mapper.UserMapper;
import com.tuoheng.admin.query.AccidentQuery;
import com.tuoheng.admin.request.accident.QueryAccidentPageListRequest;
import com.tuoheng.admin.request.inspection.QueryInspectionPageListRequest;
import com.tuoheng.admin.service.inspection.query.handle.GenerateInspectionFieldHander;
import com.tuoheng.admin.utils.CurrentUserUtil;
import com.tuoheng.admin.vo.accident.AccidentVo;
import com.tuoheng.admin.vo.accident.QueryAccidentPageVO;
import com.tuoheng.admin.vo.inspection.InspectionVo;
import com.tuoheng.common.core.config.common.CommonConfig;
import com.tuoheng.common.core.enums.ServiceExceptionEnum;
import com.tuoheng.common.core.exception.ServiceException;
import com.tuoheng.common.core.utils.DateUtils;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 查询事件分页列表业务层处理
 *
 * @author wanjing
 * @team tuoheng
 * @date 2023-03-03
 */
@Slf4j
@Service
public class QueryAccidentPageListService {

    @Autowired
    private DeptMapper deptMapper;
    @Autowired
    private AccidentMapper accidentMapper;

    public JsonResult getPageList(QueryAccidentPageListRequest request) {
//        log.info("进入查询事件分页列表业务");
        String tenantId = CurrentUserUtil.getTenantId();
        request.setTenantId(tenantId);

        JsonResult result = this.check(tenantId, request);
        if (0 != result.getCode()) {
            log.info("进入查询巡检任务分页列表业务：校验失败：{}", result.getMsg());
            return result;
        }

        List<String> deptIdList = null;
        List<Dept> deptList = deptMapper.selectList(Wrappers.<Dept>lambdaQuery()
                        .like(Dept::getName, request.getDeptName())
                        .eq(Dept::getMark, 1));
        if (CollectionUtil.isNotEmpty(deptList)) {
            deptIdList = deptList.stream().map(o -> o.getId()).collect(Collectors.toList());
        }
        request.setDeptIdList(deptIdList);

        // 设置分页参数
        IPage<Accident> page = new Page<>(request.getPage(), request.getLimit());
        IPage<Accident> pageData = accidentMapper.selectPageList(page, request);

        // 构造返回结果对象
        List<AccidentVo> accidentVoList = this.buildAccidentVoList(pageData.getRecords());

        // 重写返回结果对象
        IPage<AccidentVo> accidentVoPageData = new Page<>();
        accidentVoPageData.setPages(pageData.getPages());
        accidentVoPageData.setCurrent(pageData.getCurrent());
        accidentVoPageData.setSize(pageData.getSize());
        accidentVoPageData.setTotal(pageData.getTotal());
        accidentVoPageData.setRecords(accidentVoList);
        return JsonResult.success(accidentVoPageData);
    }

    /**
     * 检查参数
     *
     * @param tenantId
     * @param request
     * @return
     */
    private JsonResult check(String tenantId, QueryAccidentPageListRequest request) {
        return JsonResult.success();
    }

    /**
     * @param accidentList
     * @return
     */
    private List<AccidentVo> buildAccidentVoList(List<Accident> accidentList) {
        Map<String, String> deptMap = this.getDeptMap(accidentList);
        List<AccidentVo> accidentVoList = AccidentConverMapper.INSTANCE.accidentListToAccidentVoList(accidentList);
        String deptName;
        for (AccidentVo accidentVo : accidentVoList) {
            if (ObjectUtil.isNotNull(deptMap)) {
                deptName = deptMap.get(accidentVo.getDeptId());
                accidentVo.setDeptName(deptName);
            }
        }
        return accidentVoList;
    }

    /**
     *  设置列表中每一个的部门名称
     *      查询到的列表中的部门Id，作为部门id列表，查询所有的部门，该结果数据量不会太大，放到map中
     *
     * @param accidentList
     * @return
     */
    private Map<String, String> getDeptMap(List<Accident> accidentList) {
        Map<String, String> map = new HashMap<>();
        List<String> deptIdList = accidentList.stream().map(o -> o.getDeptId()).collect(Collectors.toList());
        List<Dept> deptList = deptMapper.selectListByIdList(deptIdList);
        for (Dept dept : deptList) {
            map.put(dept.getId(), dept.getName());
        }
        return map;
    }

    public JsonResult index(AccidentQuery query) {
//校验
        if(query.getLimit() == null && query.getPage() == null){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }

        //当前租户下
        String tenantId = CurrentUserUtil.getTenantId();
        if(tenantId == null){
            return JsonResult.error(AccidentEnum.TENANT_ID_IS_NULL.getCode(),AccidentEnum.TENANT_ID_IS_NULL.getMsg());
        }
        query.setTenantId(tenantId);
        IPage<Accident> page = new Page<>(query.getPage(),query.getLimit());
        IPage<QueryAccidentPageVO> pageData = new Page<>(query.getPage(),query.getLimit());
        //搜索时间
        Date startTime = null;
        Date endTime = null;
        if(StringUtils.isNotEmpty(query.getStartTime()) && StringUtils.isNotEmpty(query.getEndTime())){
            startTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,query.getStartTime()+" 00:00:00");
            endTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,query.getEndTime()+" 23:59:59");
        }
        query.setAccidentStartTime(startTime);
        query.setAccidentEndTime(endTime);
        //获取分页数据
        IPage<Accident> accidentPageData = accidentMapper.selectPage(page, Wrappers.<Accident>lambdaQuery()
                .eq(null != query.getCode(),Accident::getRoadCode,query.getCode())
                .eq(null != query.getDeptId(),Accident::getDeptId,query.getDeptId())
                .between(null != query.getAccidentStartTime() && null != query.getAccidentEndTime(),
                        Accident::getCreateTime, query.getAccidentStartTime(), query.getAccidentEndTime())
                .eq(Accident::getMark, MarkEnum.VALID.getCode())
                .orderByDesc(Accident::getCreateTime));
        //属性复制
        List<QueryAccidentPageVO> result = accidentPageData.getRecords().stream().map(x -> {
            QueryAccidentPageVO vo = new QueryAccidentPageVO();
            BeanUtils.copyProperties(x, vo);
            //公路代号
            if (StringUtils.isNotEmpty(x.getRoadCode())) {
                vo.setCode(x.getRoadCode());
            }
            //监管部门
            if (StringUtils.isNotEmpty(x.getDeptId())) {
                //根据当前部门id查询对应的部门名称
                Dept dept = deptMapper.selectById(x.getDeptId());
                vo.setName(dept.getName() !=null?dept.getName():"");
            }

            return vo;
        }).collect(Collectors.toList());
        pageData.setPages(accidentPageData.getPages());
        pageData.setRecords(result);
        pageData.setTotal(accidentPageData.getTotal());
        pageData.setCurrent(accidentPageData.getCurrent());

        return JsonResult.success(pageData);


    }
}
